#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <string.h>

#include "DspDriver.h"
#include "DebugLib.h"
#include "Constants.h"

#ifdef OSS_SOUND
#ifdef __NetBSD
#include <soundcard.h>
#else
#include <sys/soundcard.h>
#endif
#endif

int dsp_init(int dsp_file,int resolution, int channels, int rate) {
  
  int play_stereo;
  /* int parm;

     parm = BUFFER_SIZE;
     parm = 12;
     if (ioctl(dsp_file,SNDCTL_DSP_SETFRAGMENT,&parm)<0) {
     perror("Initialisation du fragment");
     close(dsp_file);
     return -1;
     } */

#ifdef OSS_SOUND
   if (ioctl(dsp_file,SOUND_PCM_RESET,0)<0) {
     warning("DspDriver.c: dsp_init: Erreur initialisation "
	     " DSP");
     close(dsp_file);
     return (-1);
    }

  if (ioctl(dsp_file,SOUND_PCM_SYNC,0)<0) {
    warning("DspDriver.c: dsp_init: Erreur lors de syncronisation du DSP");
     return (-1);
  }

   if (ioctl(dsp_file,SOUND_PCM_WRITE_BITS,&resolution)<0) {
     warning("DspDriver.c: dsp_init: Erreur lors de l'initialisation "
	     "8/16 bits");
     close(dsp_file);
    return -1;
   }

   //ioctl( dsp_file, SNDCTL_DSP_SETDUPLEX, 0 );

   // Recording parameters
   if (ioctl(dsp_file,SOUND_PCM_READ_BITS,&resolution)<0) {
  warning("DspDriver.c: dsp_init: Error");
  close(dsp_file);
    return -1;
   }
   //   if (ioctl(dsp_file,SOUND_PCM_READ_RATE,&rate)<0) {
   //  warning("DspDriver.c: dsp_init: Error");
   //  close(dsp_file);
   //  return -1;
   //}

   // finished
  play_stereo = channels - 1 ;

  if (ioctl(dsp_file,SNDCTL_DSP_STEREO,&play_stereo)<0) {
    warning("DspDriver.c: dsp_init: Erreur lors de l'initialisation"
	    " Mono/Stereo");
    close(dsp_file);
    return -1;
   }
  
  if (ioctl(dsp_file, SNDCTL_DSP_SPEED, &rate)<0) {
    warning("DspDriver.c: dsp_init: Erreur lors de l'initialisation de "
	    "la frquence");
    close(dsp_file);
    return -1;
    }
#endif
  return dsp_file;
}

int dsp_init_for_play(char *dsp_device,int resolution, int chanels, int rate) {

  int  dsp_file;
  char message[200];

  if ((dsp_file = open(dsp_device,O_WRONLY)) == -1 ) {
    strcpy(message,"DspDriver.c: dsp_init_for_play: "
	   "Error while opening ");
    strcat(message, dsp_device);
    warning(message);
    return -1;
  }
  return dsp_init(dsp_file,resolution,chanels,rate);
}

int dsp_init_for_record(char *dsp_device,int resolution, int chanels, int rate) {

  int dsp_file;

  if ((dsp_file = open(dsp_device,O_RDONLY)) == -1 ) {
    warning("DspDriver.c: dsp_init_for_record: Erreur Can't open dsp device "
	    "for read");
    return -1;
  }

  return dsp_init(dsp_file,resolution,chanels,rate);

}

int dsp_init_for_rec_and_play(char *dsp_device,int resolution, int chanels, int rate) {

  int dsp_file;

#ifdef OSS_SOUND
  if (((dsp_file = open(dsp_device,O_RDWR)) == -1 ) || 
      (ioctl( dsp_file, SNDCTL_DSP_SETDUPLEX, 0 ) < 0)) {
    warning("DspDriver.c: dsp_init_for_rec_and_play: One of your souncards is "
	    "not full-duplex. This may not be a problem.");
    if (dsp_file >= 0)
      close(dsp_file);
    return -1;
  }
#endif

  return dsp_init(dsp_file,resolution,chanels,rate);

}
 
int dsp_close(int dsp_file) {

  return close(dsp_file);

}
